@@ -20,7 +20,7 @@ from base64 import b64encode, b64decode
 from fair_research_login.client import NativeClient
 from mdml_client.config import CLIENT_ID
 from uuid import uuid4
-from confluent_kafka import Consumer
+from confluent_kafka import Consumer, Producer
 from confluent_kafka import SerializingProducer
 from confluent_kafka.serialization import StringSerializer
 from confluent_kafka.schema_registry import SchemaRegistryClient
@@ -189,6 +189,13 @@ def chunk_file(fn, chunk_size, use_b64=True, encoding='utf-8'):
         part += 1
         yield dat
 
+py_type_to_schema_type = {
+    str: "string",
+    float: "number",
+    int: "number",
+    list: "array"
+}
+
 def create_schema(d, title, descr, required_keys=None):
     """
     Input data object is turned into a schema for use
@@ -214,22 +221,68 @@ def create_schema(d, title, descr, required_keys=None):
     }
     if required_keys is not None:
         schema['required'] = required_keys
-    keys = d.keys()
-    print(keys)
-    for key in keys:
-        dtype = type(d[key])
-        if dtype == str:
+    for key in d.keys():
+        try:
+            dtype = py_type_to_schema_type[type(d[key])]
+        except:
+            raise Exception("Unhandled type exception")
+        if dtype == "array":
+            item_type = py_type_to_schema_type[type(d[key][0])]
             schema['properties'][key] = {
-                "type": "string"
-            }
-        elif dtype == float or dtype == int:
-            schema['properties'][key] = {
-                "type": "number"
+                "type": "array",
+                "items": {
+                    "type": item_type
+                }
             }
         else:
-            raise Exception(f"Data is currently using an unsupported type: {dtype}")
+            schema['properties'][key] = {
+                "type": dtype
+            }
     return schema
 
+class kafka_mdml_producer_schemaless:
+    """
+    Creates a Producer instance for interacting with the MDML
+
+    Parameters
+    ----------
+    topic : str
+        Topic to send under
+    kafka_host : str
+        Host name of the kafka broker
+    kafka_port : int
+        Port used for the Kafka broker
+    """
+    def __init__(self, topic, kafka_host="merf.egs.anl.gov", kafka_port=9092):
+        # Checking topic param
+        if type(topic) == str:
+            if topic[0:5] != "mdml-":
+                raise Exception("Error, topic must be of the form 'mdml-<experiment id>-<sensor>'")
+            else:
+                self.topic = topic
+        else:
+            raise Exception("Error, topic must be of type string.")
+        # Create producer and its config 
+        producer_config = {
+            'bootstrap.servers': f'{kafka_host}:{kafka_port}'
+        }
+        self.producer = Producer(producer_config)
+    def produce(self, data):
+        """
+        Produce data to the supplied topic 
+
+        Parameters
+        ----------
+        data : dict
+            Dictionary of the data
+        """
+        self.producer.produce(topic=self.topic, value=data)
+    def flush(self):
+        """
+        Flush (send) any messages currently waiting in the producer.
+        """
+        self.producer.flush()
+
 class kafka_mdml_producer:
     """
     Creates a serializingProducer instance for interacting with the MDML. 
@@ -281,7 +334,7 @@ class kafka_mdml_producer:
             'value.serializer': json_serializer
         }
         self.producer = SerializingProducer(producer_config)
-    def produce(self, data):
+    def produce(self, data, key="key"):
         """
         Produce data to the supplied topic 
 
@@ -290,7 +343,7 @@ class kafka_mdml_producer:
         data : dict
             Dictionary of the data
         """
-        self.producer.produce(topic = self.topic, value=data)
+        self.producer.produce(topic = self.topic, key=key, value=data)
     def flush(self):
         """
         Flush (send) any messages currently waiting in the producer.
@@ -458,6 +511,68 @@ class kafka_mdml_consumer:
             except KeyboardInterrupt:
                 break
 
+class kafka_mdml_consumer_schemaless:
+    """
+    Creates a serializingProducer instance for interacting with the MDML. 
+    
+    Parameters
+    ----------
+    topics : list(str)
+        Topics to consume from 
+    group : str
+        Consumer group ID. Messages are only consumed by a given group ID
+        once.
+    kafka_host : str
+        Host name of the kafka broker
+    kafka_port : int
+        Port used for the kafka broker
+    """
+    def __init__(self, topics, group, 
+                kafka_host="merf.egs.anl.gov", kafka_port=9092):
+        self.topics = topics
+        self.group = group
+        self.kafka_host = kafka_host
+        self.kafka_port = kafka_port
+        # Checking topic param
+        if type(topics) == list:
+            for topic in topics:
+                if type(topic) == str:
+                    if topic[0:5] != "mdml-":
+                        raise Exception("Error, topic must be of the form 'mdml-<experiment id>-<sensor>'")
+                else:
+                    raise Exception("Error, topic must be of type string.")
+        else:
+            raise Exception("Error, topics parameter must be a list of strings.")
+        
+        consumer_conf = {
+            'bootstrap.servers': f"{kafka_host}:{kafka_port}",
+            'group.id': group,
+            'auto.offset.reset': 'earliest',
+            'allow.auto.create.topics': 'true' # prevents unknown topic error 
+        }
+        consumer = Consumer(consumer_conf)
+        consumer.subscribe(topics)
+        self.consumer = consumer
+
+    def consume(self, poll_timeout=1.0, overall_timeout=300.0):
+        print(f"Consumer loop will exit after {overall_timeout} seconds without receiving a message or with Ctrl+C")
+        timeout = 0.0
+        while timeout < overall_timeout:
+            try:
+                msg = self.consumer.poll(poll_timeout)
+                if msg is None:
+                    timeout += poll_timeout
+                    continue # no messages within timeout - poll again 
+                timeout = 0.0
+                yield {
+                    'topic': msg.topic(),
+                    'value': msg.value()
+                }
+            except KeyboardInterrupt:
+                break
+        
+
+
 class kafka_mdml_s3_client:
     """
     Creates an MDML producer for sending >1MB files to an s3 location. Simultaneously, the MDML sends 
