@@ -1,6 +1,6 @@
 from .MDML_client import *
 name = "MDML_Client"
-__version__ = "1.1.88"
+__version__ = "1.1.93"
 multipart_schema = {
     "$schema": "http://merf.egs.anl.gov/mdml-multipart-message-schema#",
     "title": "MultipartMessageSchema",
@@ -26,3 +26,16 @@ multipart_schema = {
     },
     "required": [ "chunk", "part", "filename", "encoding" ]
 }
+stop_funcx_schema = {
+    "$schema": "http://merf.egs.anl.gov/mdml-example-stop-funcx-schema#",
+    "title": "StopFuncx",
+    "description": "Schema for the message that will stop a FuncX function",
+    "type": "object",
+    "properties": {
+        "stop": {
+            "description": "ID of the data point",
+            "type": "boolean"
+        }
+    },
+    "required": [ "stop" ]
+}
